@@ -1,7 +1,9 @@
 #!/usr/bin/env python2
 
-from SettingsWidgets import *
+import gi
+gi.require_version("Gtk", "3.0")
 from gi.repository import Gtk, CDesktopEnums
+from GSettingsWidgets import *
 
 DPI_FACTOR_LARGE         = 1.25
 DPI_FACTOR_NORMAL        = 1.0
@@ -12,6 +14,7 @@ KEY_GTK_THEME            = "gtk-theme"
 KEY_ICON_THEME           = "icon-theme"
 KEY_WM_THEME             = "theme"
 
+
 class Module:
     name = "universal-access"
     comment = _("Configure accessibility features")
@@ -42,7 +45,7 @@ class Module:
 
             settings = page.add_section(_("Visual Aids"))
 
-            switch = GSettingsSwitch(_("High contrast"))
+            switch = Switch(_("High contrast"))
             self.iface_settings.bind_with_mapping(KEY_GTK_THEME,
                                                   switch.content_widget, "active",
                                                   Gio.SettingsBindFlags.DEFAULT,
@@ -50,7 +53,7 @@ class Module:
                                                   self.hi_con_set_mapping)
             settings.add_row(switch)
 
-            switch = GSettingsSwitch(_("Large text"))
+            switch = Switch(_("Large text"))
             self.iface_settings.bind_with_mapping(KEY_TEXT_SCALING_FACTOR,
                                                   switch.content_widget, "active",
                                                   Gio.SettingsBindFlags.DEFAULT,
@@ -62,6 +65,7 @@ class Module:
                                      "org.cinnamon.desktop.a11y.applications",
                                      "screen-reader-enabled",
                                      None,
+                                     ["orca"],
                                      ["gnome-orca"])
             settings.add_row(switch)
 
@@ -148,6 +152,12 @@ class Module:
             widget = GSettingsComboBox(_("Keyboard layout"), "org.cinnamon.keyboard", "keyboard-type", keyboard_type_options)
             settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-keyboard-enabled")
 
+            activation_mode_options = [["accessible", _("Show the keyboard any time something expects input")],
+                                       ["on-demand",  _("Show keyboard only when the user activates it")]];
+
+            widget = GSettingsComboBox(_("Activation mode"), "org.cinnamon.keyboard", "activation-mode", activation_mode_options)
+            settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-keyboard-enabled")
+
 # Keyboard indicators
 
             settings = page.add_section(_("Keyboard indicators"))
@@ -341,9 +351,9 @@ class Module:
             install_widget = SettingsWidget()
 
             self.dep_button = DependencyCheckInstallButton(_("Checking dependencies"),
-                                                           _("Install: %s") % ("mousetweaks"),
+                                                           _("Please install: %s") % ("mousetweaks"),
                                                            ["mousetweaks"],
-                                                           None,
+                                                           Gtk.Alignment(),
                                                            self.on_dep_satisfied)
 
             install_widget.pack_start(self.dep_button, True, False, 0)
